﻿using System;
using System.IO;
using Ozeki.Media.MediaHandlers;

namespace DTMF_IVR_WinForm.Model
{
    /// <summary>
    /// This IVR menu class records a voice message in uncrompressed wav format.
    /// </summary>
    class IVRMenuVoiceMessageRecorder : IVRMenuBase
    {
        private string postIntroduction = "Your message successfully stored. Thank you.";

        /// <summary>
        /// The object that reads the introduction.
        /// </summary>
        private TextToSpeech txtToSpeech;

        /// <summary>
        /// The recorder object
        /// </summary>
        private WaveStreamRecorder waveStreamRecorder;

        public IVRMenuVoiceMessageRecorder(IVRMenuElement menu)
            : base(menu)
        {
            string[] s = Introduction.Split('|');
            if (s.Length != 2)
                return;
            Introduction = s[0];
            postIntroduction = s[1];
        }


        public override void StartIntroduction()
        {
            txtToSpeech = new TextToSpeech(new Ozeki.VoIP.Media.AudioFormat(8000, 1, 16, 20));
            txtToSpeech.Stopped += new EventHandler<EventArgs>(txtToSpeech_Stopped_Introduction);
            OnIntroductionStarting(txtToSpeech);
            txtToSpeech.AddText(Introduction);
            txtToSpeech.StartStreaming();
        }

        public override void StopIntroduction()
        {
            CloseRecorder();
            CloseTextToSpeach();
        }

        public override void RestartIntroduction()
        { }

        public override void CommandReceived(int signal)
        {
            switch (signal)
            {
                case 0:
                    CloseRecorder();
                    OnStepIntoMenu(null);
                    return;
                case 10:
                    CloseRecorder();
                    CloseTextToSpeach();
                    OnStepIntoMenu(Parent);
                    return;
                case 11:
                    waveStreamRecorder.StopStreaming();
                    CloseRecorder();
                    txtToSpeech.Stopped += new EventHandler<EventArgs>(txtToSpeech_Stopped_postIntroduction);
                    txtToSpeech.AddText(postIntroduction);
                    txtToSpeech.StartStreaming();
                    return;
            }
        }

        /// <summary>
        /// Indicates the post message has read. The call ends.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void txtToSpeech_Stopped_postIntroduction(object sender, EventArgs e)
        {
            txtToSpeech.Stopped -= txtToSpeech_Stopped_postIntroduction;
            OnIntroductionStoped(txtToSpeech);
            txtToSpeech.Dispose();
            OnStepIntoMenu(null);
        }

        /// <summary>
        /// Indicates the introduction has finished. It's starts the voice record.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void txtToSpeech_Stopped_Introduction(object sender, EventArgs e)
        {
            txtToSpeech.Stopped -= txtToSpeech_Stopped_Introduction;
            waveStreamRecorder = new WaveStreamRecorder(Path.Combine(Environment.CurrentDirectory, string.Format("IVRVoiceMesage{0}.wav", DateTime.Now.ToFileTime())));

            OnIntroductionStarting(waveStreamRecorder);
            waveStreamRecorder.StartStreaming();
        }


        /// <summary>
        /// Releases the recorder object
        /// </summary>
        void CloseRecorder()
        {
            if (waveStreamRecorder != null)
            {
                waveStreamRecorder.StopStreaming();
                OnIntroductionStoped(waveStreamRecorder);
                waveStreamRecorder.StopStreaming();
                waveStreamRecorder.Dispose();
            }
        }

        /// <summary>
        /// Release the reader object.
        /// </summary>
        void CloseTextToSpeach()
        {
            if (txtToSpeech!=null)
            {
                txtToSpeech.Stopped -= txtToSpeech_Stopped_Introduction;
                txtToSpeech.Stopped -= txtToSpeech_Stopped_postIntroduction;
                txtToSpeech.StopStreaming();
                txtToSpeech.Dispose();
                OnIntroductionStoped(txtToSpeech);
            }
        }

    }
}
